
C User may use this to test the Srem2d.f subroutine 
C against the standard set of reference and satellite rainfall
C data (and error parameters provided herein - refer to IEEE paper
C where error calibration is documented

C Date: Nov 12, 2008 for PMM project - for tRIBS srem2d for Thymios
C For generation of ensembles the forward way using
C calibrated SREM2D parameters

      parameter(nx=106,ny=96,ntime=365,niter=50)
      real raing(nx,ny,ntime),rains(nx,ny,ntime)
      real raingg(nx,ny,ntime)
      real raindet_slopea,raindet_slopeb
      real false, bmean, sig, sigjit, brain
      real bret,p0

      character*20 cmd

      open(10,file='Rainfall_0506_3B42.txt',status='old')

C Name a file to write your SREM2D simulations
      open(19,file='rainfallorig.dat',status='unknown')
      open(21,file='param_3b42lp_inv1.dat',
     1status='old')
 
      iseedjit=-499      
      iseedbr=75
      iseedbnr=65
      iseedret=-399 

      read(21,*)raindet_slopea
      read(21,*)raindet_slopeb
      read(21,*)false
      read(21,*)bmean
      read(21,*)sig
      read(21,*)brain
      read(21,*)bnorain
      read(21,*)bret
      read(21,*)rho
      read(21,*)p0
      close(21)

      dx=25.0
      dy=25.0
      nxx=nx
      nyy=ny
      maxln=ntime

C Read true rainfall data
C 'nexrain_5to8.srem2d' has reference(true) rainfall
C from NEXRAD from May 1 to August 30, 2002
C Total of 2952 timesteps at hourly resolution

         do isat=1,maxln
         do ix=1,nyy
         read(10,*)(raing(jx,ix,isat),jx=1,nxx)
         enddo

         do ix=1,nyy
         do jx=1,nxx
         if(raing(jx,ix,isat).lt.0)then
         raing(jx,ix,isat)=0.0
         endif
         enddo
         enddo

         read(10,*)
        enddo

        close(10)
 11     format(106(f8.3,2x))
        
       do iter=1,niter
       
       print*,'iter  ',iter

      Call Srem2d(raing,raingg,rains,nxx,nyy,brain,bnorain,bret,
     1bmean,rho,p0,false,sig,raindet_slopea,raindet_slopeb,
     1maxln,iseedbr,iseedbnr,iseedret,dx,dy)

       write(19,*)iter

       do isave=1,maxln
       do isavey=1,nyy
       write(19,11)(rains(isavex,isavey,isave),isavex=1,nxx)
       enddo
       write(19,*)
       enddo

      
       enddo


       end

       Subroutine Srem2d(raing,raingg,rains,nxx,nyy,brain,
     1bnorain,bret,bmean,rho,p0,false,sig,rain_det_slopea,
     1rain_det_slopeb,maxln,iseedbr,
     1iseedbnr,iseedret,dxx,dyy)

C SREM-2D for generating correlated random fields of rain
C retrievals
C Refer to Manual last updated on January 4, 2004 for details

C raing: true rainfield in nxx,nyy dimensions - rain is accumulated
C        hourly rainfall in mm/hr
C rains:simulated satellite instantaneous rainrate field (mm/hr)

C maxln: maximum number of timesteps in an event

C nxx: max dimension in 'i' - x dimension
C nyy: max dimension in 'j' - y dimension

C brain= correlation distance for rain detection (km)
C bnorain= correlation distance for no rain detection (km)
C bret=correlation distance for rain retrieval (km)

C iseedjit=seed for Bernoulli trials for jitter (-ve integer)
C iseedbr=seed for Bernoulli trials for rain (+ve integer)
C iseedbnr=seed for Bernoulli trials for no rain (+ve integer)
C iseedret=seed for Bernoulli trials for retrieval (-ve integer)

C sig=standard deviation of retrieval (Gaussian error)
C bmean = mean of retrieval (Gaussian error)
C rho= temporal correlation (not the square)

C rain_det_slopea- the a parameter of the sigmoidal function
C rain_det_slopeb - the b parameter of the sigmoidal function
C slopeajit - the a parameter for jitter (from accu to instantaneous)
C slopebjit - the b parameter for jitter (from accu to instantaneous) 

C nxdim=Grid dimension in x (km)
C nydim=Grid dimension in y (km)
C dxx=Grid mesh dimension in x direction
C dyy=Grid mesh dimension in y direction

C Note: Covariance function is fixed as an Exponential model

C Program Modfied by Faisal Hossain
C Last Modified: Jan 31, 2004, UConn

       real zfld(nxx,nyy),raing(nxx,nyy,maxln)
       real raingg(nxx,nyy,maxln)
       real etar(nxx,nyy),etanr(nxx,nyy),etaret(nxx,nyy)
       real rains(nxx,nyy,maxln)
       real mu,sig,dx,dy
       integer abl(maxln),bbl(maxln)

        tau=0.00

C May 22, 2008
C More intelligent Constraint added to prevent occasional
C high bias or weird high rainfall simulations
C IN this case, it is ensured that the simulated rain never
C exceeds a threshold. 
C The threshold is 1.25*expectedbias

        expectedbias=exp(bmean+(0.5*sig*sig))
        threshold=1.25*expectedbias

        a=rain_det_slopea
        b=rain_det_slopeb
        c=false

        nxsize=nxx
        nysize=nyy 

        nxdim=nxsize
        nydim=nysize

        n_rain=maxln

C STEP 1:  Make Instantaneous true rainrates

C 11/11/04: (Note for OK study we dont' need this
C step as we calibrated consistently (NEXRAD-HUFFMAN)
C and are applying it consistently as well

      do isat=1,n_rain

      do j=1,nyy
       do i=1,nxx

            raingg(i,j,isat)=raing(i,j,isat)

          enddo
         enddo
       enddo     

C Start your time series here:

       bmcor=bmean
       sigx=sig

       ngr=0
       ngrt=0
       do isat=1,n_rain

C Generate three random fields, one each with brain, bnorain
C and bret correlation lengths (in km)
C The random fields are generated from Turning bands Method
C The detection fields are converted to 0-1 numbers by the Error Function
C Transformation. Insignificant change in correlation length
C is assumed at low lags.
C The retrieval field is not transformed
     
C RAIN DETECTION
  
      call trngbnd(brain,nxdim,nydim,dxx,dyy,iseedbr,zfld,
     1nxsize,nysize)

      do j=1,nyy
        do i=1,nxx
         xxu=zfld(i,j)/sqrt(2.0)
         etar(i,j)=(0.5+(0.5*erf(xxu)))
        enddo
      enddo

      Call adjustt(etar,nxsize,nysize)

C NO RAIN DETECTION

      call trngbnd(bnorain,nxdim,nydim,dxx,dyy,iseedbr,zfld,
     1nxsize,nysize)

      do j=1,nyy
        do i=1,nxx
         xxu=zfld(i,j)/sqrt(2.0)
         etanr(i,j)=(0.5+(0.5*erf(xxu)))
        enddo
      enddo

      call adjustt(etanr,nxsize,nysize)

C RAIN RETRIEVAL

C Generate a field of random Gaussian fields for rain retrieval
      
      call trngbnd(bret,nxdim,nydim,dxx,dyy,iseedret,zfld,
     1nxsize,nysize)

      bmtot=0.0
      do j=1,nyy
        do i=1,nxx
         etaret(i,j)=(zfld(i,j)*sigx)+bmcor
         bmtot=etaret(i,j)+bmtot
        enddo
      enddo

      bmcor=bmtot/real(nxx*nyy)

C STEP 2: Rain and No Rain Detection

       do j=1,nyy
        do i=1,nxx


       if(raingg(i,j,isat).gt.tau) then
c        ngrt=ngrt+1
C  Compute P1 first

	    p1=1.0/(a+(exp(-1.*raingg(i,j,isat)*b)))
            pp=etar(i,j)       
            if(pp.le.p1)then
            rains(i,j,isat)=exp(log(raingg(i,j,isat))
     1      +etaret(i,j))
                       
C Modification on March 27, 2008: Guard against unusual high rainfall values
C Replace the high values with reference when rbias=3.0

            rbias=rains(i,j,isat)/raingg(i,j,isat)
            
            if(rbias.gt.threshold)then
            ngr=ngr+1
            rains(i,j,isat)=raingg(i,j,isat)*expectedbias
            endif

            else
            rains(i,j,isat)=0.0
            endif

       else

C Condition for raing being == 0.0
            
            pp=etanr(i,j)

            if(pp.le.p0)then
            rains(i,j,isat)=0.0
            else
 1          dumexp=ran1(iseedret)
            if(dumexp.eq.0.0) goto 1  
            rains(i,j,isat)=-log(dumexp)/c
            endif
        endif

          enddo
       enddo

C  Do time correlation of etaret
	    	    
            bmcor=bmean+(rho*(bmcor-bmean))
	    sigx=(sig*sig)*(1.-(rho*rho))
	    sigx=sqrt(sigx)
         
C End timestep loop
      
       enddo	        
       print*,ngr
       end
      
        Subroutine origin(rmvv,nss,origg,rangg)
        real rmvv(nss),origg,rangg
        integer nss

C Find the minimum value of rmvv

        origg=1000000.0
        do i=1,nss
        if(rmvv(i).lt.origg) origg=rmvv(i)
        enddo

C Find the maximum value of rmvv

        rmax=0.0
        do i=1,nss
        if(rmvv(i).gt.rmax) rmax=rmvv(i)
        enddo

        rangg=abs(rmax-origg)

        return

        end

        Subroutine meanvar(rmvv,nss,rmm,rvv)

        real rmvv(nss),rmm,rvv,orig
        real rmvs(nss)

        rmtot=0.0
        do i=1,nss
        rmtot=rmtot+rmvv(i)
        enddo
        rmm=rmtot/real(nss)

        rmtot=0.0
        do i=1,nss
        rmtot=rmtot+((rmvv(i)-rmm)**2)
        enddo
        rvv=sqrt(rmtot/real(nss))

        return
        end


C*    ***** subroutine GENERATES SECOND ORDER STATIONARY FIELDS *****
C*    *****************************************************************
C*    *                                                               *
C*    *             developed from PROGRAM STAGER                     *
C*    *                                                               *
C*    *    PROGRAM GENERATES SECOND ORDER STATIONARY RANDOM FIELDS    *
C*    *                                                               *
C*    *   original AUTOR: WITOLD F. KRAJEWSKI         NOVEMBER 1984   *
C*    *                                                               *
C*    *   a dimensioning error in stager was corrected by c. cong     *
C*    *                                                               *
C*    *   j. schaake organized subroutine structure    
C*    * 
C*    *   Modified by t.bellerby
C*    *                                                               *
C*    *   date last modified:  june, 2003                             *
C*    *                                                               *
C*    *****************************************************************
C*
C*    *** subroutine INPUT:
C*
C*             Icov   - COVARIANCE FUNCTION CODE:
C
C*                            1 - EXPONENTIAL
C*                            2 - BESSEL TYPE
C*                            3 - DOUBLE EXPONENTIAL
c
c              cdist  - correlation distance of the field
C*             NXdim  - GRID DIMENSION OF THE FIELD IN X DIRECTION
C*             NYdim  - GRID DIMENSION OF THE FIELD IN Y DIRECTION
C*             DXx    - GRID MESH DIMENSION IN X DIRECTION (km)
C*             DYy    - GRID MESH DIMENSION IN Y DIRECTION (km)
C*             iseed  - RANDOM NUMBER GENERATOR SEED USED TO GENERATE
c
c    **** definitions of internal variables
c
c                  ityp   - covariance function code
C*                 CORDIS - CORRELATION DISTANCE IN THE FIELD
C*                 NX     - GRID DIMENSION OF THE FIELD IN X DIRECTION
C*                 NY     - GRID DIMENSION OF THE FIELD IN Y DIRECTION
C*                 DX     - GRID MESH DIMENSION IN X DIRECTION (km)
C*                 DY     - GRID MESH DIMENSION IN Y DIRECTION (km)
C*                 LINES  - NUMBER OF LINES IN TBM
C*                 NHARM  - NUMBER OF HARMONICS IN TBM
C*

      SUBROUTINE trngbnd (CDIST,NXDIM,NYDIM,DXX,DYY,iseed,zfield,
     1nxsize,nysize)

      COMMON/TBMD/NX,NY,DX,DY,LINES,NHARM,CORPAR
      real zfield(nxsize,nysize)
      EXTERNAL SDF
C*
      
      CORDIS = CDIST
      NX = NXDIM
      NY = NYDIM
      DX = DXX
      DY = DYY
      CORPAR = 1./CORDIS
C*

C* Note: If you want to modify the covariance function then
C modify the SDF*** function routine at the end of the program
C and make that SDF below for Field routine.

        CALL FIELD(SDF,ISEED,zfield,nxsize,nysize)

      return
      end
C*    ******************************************************************
      SUBROUTINE FIELD(SDF,ISEED,zfield,nxsize,nysize)
C *    ******************************************************************
      
      COMMON/TBMD/NX,NY,DX,DY,LINES,NHARM,CORPAR
      real zfield(nxsize,nysize)
      EXTERNAL SDF
C*
C*    *** SUBROUTINE COMPUTES VALUES OF
C         A homogeneous, STATIONARY RANDOM FIELD WITH
C*        zero MEAN, unit VARIANCE, AND specified CORRELATION FUNCTION
C*
C*              INPUT arguments:
c
c                      SDF   - TWO DIMENSIONAL SPECTRAL DENSITY FUNCTION
C*                     ISEED - RUNDOM NUMBER SEED
C*
      parameter (maxlines=64)
      parameter (maxharm=800)
      real vfee(maxharm)
      real sdfa(maxharm)
      real antan(maxlines)
      parameter (maxud=1000)
      COMMON/BIG/UD(maxlines,-maxud:maxud)
      real rlines,omega,omk
c
      LINES = maxlines
      NHARM = maxharm
C*
C*    *** DETERMINE SPECTRUM RANGE AND DISCRETIZATION ALONG THE LINES
C*
      discr = sqrt(dx**2+dy**2)/5.
C*
C*    *** DETERMINE THE ANGLE BETWEEN LINES
C*
      PI=3.141592654
      ANSTEP=PI/FLOAT(LINES)
      RLINES=SQRT(FLOAT(LINES))
C*
C*    *** DETERMINE THE LENGTH OF A VECTOR TO STORE LINE PROCESS
C*
      ANGLE=0.0
      diagonal = sqrt((nx*dx)**2+(ny*dy)**2)
      NDISCR=IFIX(diagonal/DISCR)+5
      if (ndiscr.gt.maxud) then
        ndiscr = maxud
        discr =  diagonal/(ndiscr-5)
      endif
c
c     *** process the lines ***
c
      OMEGA=40.0*CORPAR
      DELTA0=OMEGA/NHARM
C*
C*    *** GENERATE LINE PROCESS VALUE
C*
      DO K=1,NHARM
        OMK=(K-0.5)*DELTA0
        SDFA(K)=SQRT(SDF(OMK,CORPAR)*DELTA0)
      enddo

      DO L=1,LINES
        ZP1=TAN(ANGLE)
        antan(l)=zp1
        DO K=1,NHARM
          fee=ran1(iseed)
          VFEE(K)=2.0*PI*FEE
        enddo
        PSI=0.5*DISCR
        DO J=-NDISCR,NDISCR
          UD(L,J)=UPGEN(PSI,NHARM,SDFA,CORPAR,VFEE)
          PSI=PSI+DISCR
        enddo
        ANGLE=ANGLE+ANSTEP
      enddo
C*
C*    *** PROJECT THE POINTS ON THE LINES AND ASSIGN VALUES OF THE FIELD
C*
      DO J=1,NY
        ZPY=(j-ny)*DY
        DO I=1,NX
          ZPX=(I-1)*DX
          C2=ZPX*ZPX+ZPY*ZPY
          S=0.0
          DO L=1,LINES
            ZP1=antan(l)
            A=(ZPX*ZP1+ZPY)**2
            A2=A/(ZP1*ZP1+1.0)
            PSI=SQRT(A2)
            intpsi=ifix(psi/discr)
            S=S+UD(L,INTPSI)
          enddo
          ZFIELD(I,J)=S/RLINES
        enddo
      enddo
      RETURN
      END
C*    ******************************************************************
      FUNCTION UPGEN(X,NHARM,SDFA,CORPAR,VFEE)
C*    ******************************************************************
      real vfee(*)
      EXTERNAL SDF
C*
C*    *** FUNCTION GENERATES UNIDIMENSIONAL PROCESS WITH GIVEN
C*        CORRELATION PARAMETER
C*
C*        INPUT:
C*                      X      - INDEPENDENT VARIABLE
C*                      NHARM  - NUMBER OF HARMONICS USED TO GENERATE LIN
C*                               PROCESS
C*                      SDF    - TWO DIMENSIONAL SPECTRAL DENSITY FUNCTIO
C*                      VAR    - VARIANCE OF THE PROCESS
C*                      CORPAR - CORRELATION PARAMETER
C*                      VFEE   - VECTOR OF RANDOM NUMBERS
C*
c
c         INTERNAL VARIABLES:
C*                      OMEGA  - MAXIMUM RANGE OF SPECTRAL DENSITY FUNCTI

      real sdfa(NHARM)
      S=0
      OMEGA=40.0*CORPAR
      DELTAO=OMEGA/NHARM
C*
C*    *** GENERATE LINE PROCESS VALUE
C*
      DO K=1,NHARM
        OMK=(K-0.5)*DELTAO
        S=S+SDFA(k)*COS(omk*X+VFEE(K))
      enddo
      UPGEN=2.0*S
      RETURN
      END
C*    ******************************************************************
      FUNCTION SDF(X,CORPAR)
C*    FUNCTION SDFEXP(X,CORPAR)
C*    ******************************************************************
C*
C*    *** FUNCTION COMPUTES VALUES OF SPECTRAL DENSITY FUNCTION
C*        FOR EXPONENTIAL COVARIANCE FUNCTION
C*
C*        PARAMETERS:
C*              INPUT:  X      - INDEPENDENT VARIABLE
C*                      CORPAR - CORRELATION PARAMETER
C*
      ZP1=CORPAR*CORPAR
      SDF=0.5*X/(ZP1*(1.0+X*X/ZP1)**1.5)
      RETURN
      END
C*    ******************************************************************
      FUNCTION SDFBES(X,CORPAR)
C*    ******************************************************************
C*
C*    *** FUNCTION COMPUTES VALUES OF SPECTRAL DENSITY FUNCTION
C*        FOR BESSEL TYPE COVARIANCE FUNCTION
C*
C*        PARAMETERS:
C*              INPUT:  X      - INDEPENDENT VARIABLE
C*                      CORPAR - CORRELATION PARAMETER
C*
      ZP1=CORPAR*CORPAR
      ZP2=X*X
      ZP3=1.0+ZP2/ZP1
      SDFBES=X/(ZP1*ZP3*ZP3)
      RETURN
      END
C*    ******************************************************************
      FUNCTION SDFDOU(X,CORPAR)
C*    ******************************************************************
C*
C*    *** FUNCTION COMPUTES VALUES OF SPECTRAL DENSITY FUNCTION
C*        FOR DOUBLE EXPONENTIAL COVARIANCE FUNCTION
C*
C*        PARAMETERS:
C*              INPUT:  X      - INDEPENDENT VARIABLE
C*                      CORPAR - CORRELATION PARAMETER
C*
      ZP1=4.0*CORPAR*CORPAR
      SDFDOU=X*EXP(-X*X/ZP1)/ZP1
      RETURN
      END

c=============================================================
c     Random number generator
c=============================================================
      FUNCTION ran1(idum)
      INTEGER idum,IA,IM,IQ,IR,NTAB,NDIV
      REAL ran1,AM,EPS,RNMX
      PARAMETER (IA=16807,IM=2147483647,AM=1./IM,IQ=127773,IR=2836,
     *NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
      INTEGER j,k,iv(NTAB),iy
      SAVE iv,iy
      DATA iv /NTAB*0/, iy /0/
      if (idum.le.0.or.iy.eq.0) then
        idum=max(-idum,1)
        do 11 j=NTAB+8,1,-1
          k=idum/IQ
          idum=IA*(idum-k*IQ)-IR*k
          if (idum.lt.0) idum=idum+IM
          if (j.le.NTAB) iv(j)=idum
11      continue
        iy=iv(1)
      endif
      k=idum/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum=idum+IM
      j=1+iy/NDIV
      iy=iv(j)
      iv(j)=idum
      ran1=min(AM*iy,RNMX)
      return
      END

      integer function lb(string)
      character*(*) string

      j=1
      do i=1,len(string)
         if(string(i:i).ne.' ') j=i
      enddo
      lb=j
      end



      FUNCTION erf(x)

      REAL erf,x
CU    USES gammp
      REAL gammp
      if(x.lt.0.)then
        erf=-gammp(.5,x**2)
      else
        erf=gammp(.5,x**2)
      endif
      return
      END

C  (C) Copr. 1986-92 Numerical Recipes Software .1"1$9.

      FUNCTION gammp(a,x)
      REAL a,gammp,x
CU    USES gcf,gser
      REAL gammcf,gamser,gln
      if(x.lt.0..or.a.le.0.)pause 'bad arguments in gammp'
      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software .1"1$9.

      SUBROUTINE gser(gamser,a,x,gln)
      INTEGER ITMAX
      REAL a,gamser,gln,x,EPS
      PARAMETER (ITMAX=100,EPS=3.e-7)
CU    USES gammln
      INTEGER n
      REAL ap,del,sum,gammln
      gln=gammln(a)
      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in gser'
        gamser=0.
        return
      endif
      ap=a
      sum=1./a
      del=sum
      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gser'
1     gamser=sum*exp(-x+a*log(x)-gln)
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software .1"1$9.

      SUBROUTINE gcf(gammcf,a,x,gln)
      INTEGER ITMAX
      REAL a,gammcf,gln,x,EPS,FPMIN
      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)
CU    USES gammln
      INTEGER i
      REAL an,b,c,d,del,h,gammln
      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d
      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b
        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c
        if(abs(c).lt.FPMIN)c=FPMIN
        d=1./d
        del=d*c
        h=h*del
        if(abs(del-1.).lt.EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gcf'
1     gammcf=exp(-x+a*log(x)-gln)*h
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software .1"1$9.

      FUNCTION gammln(xx)
      REAL gammln,xx
      INTEGER j
      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)
      SAVE cof,stp
      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      gammln=tmp+log(stp*ser/x)
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software .1"1$9.


        Subroutine adjustt(zfldt,nx,ny)
        real zfldt(nx,ny),rmv(nx*ny)

C handles a maximum of nx*ny pixels

C Now normalize zfld by its mean and variance (technically
C the mean is now 0.5 and the variance is 0.0833

C After normalization, shift the origin to the minimum
C value of zfld


       id=0
        do ix=1,ny
           do iy=1,nx
            id=id+1
        rmv(id)=zfldt(iy,ix)
            enddo
           enddo
         ns=id

        call meanvar(rmv,ns,rm,rv)

       do ix=1,ny
           do iy=1,nx
        zfldt(iy,ix)=(zfldt(iy,ix)-rm)/rv
            enddo
           enddo

       id=0
        do ix=1,ny
           do iy=1,nx
            id=id+1
        rmv(id)=zfldt(iy,ix)
            enddo
           enddo
         ns=id

        call origin(rmv,ns,orig,range)

          do ix=1,ny
           do iy=1,nx
        zfldt(iy,ix)=((zfldt(iy,ix)-orig))/range
            enddo
           enddo

         return
          end


      FUNCTION gasdev(idum)
      INTEGER idum
      REAL gasdev
CU    USES ran1
      INTEGER iset
      REAL fac,gset,rsq,v1,v2,ran1
      SAVE iset,gset
      DATA iset/0/
      if (iset.eq.0) then
1       v1=2.*ran1(idum)-1.
        v2=2.*ran1(idum)-1.
        rsq=v1**2+v2**2
        if(rsq.ge.1..or.rsq.eq.0.)goto 1
        fac=sqrt(-2.*log(rsq)/rsq)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      END


